@@ -798,7 +798,7 @@ static const SECTION union { const guint8 data[6308]; const double alignment; vo
   0x03, 0x00, 0x00, 0x00
 } };
 
-static GStaticResource static_resource = { cc_background_resource_data.data, sizeof (cc_background_resource_data.data), NULL, NULL, NULL };
+static GStaticResource static_resource = { cc_background_resource_data.data, sizeof (cc_background_resource_data.data) };
 extern GResource *cc_background_get_resource (void);
 GResource *cc_background_get_resource (void)
 {
