@@ -23,13 +23,10 @@
 
 #include <glib.h>
 
-typedef struct {
-  const gchar *value;
-  const gchar *description;
-} CcInputSwitcherOptions;
-
-extern CcInputSwitcherOptions cc_input_switcher_options[];
-
 char *cc_util_normalize_casefold_and_unaccent (const char *str);
 
+const gchar *cc_util_xkb_info_description_for_option (void        *info,
+                                                      const gchar *group_id,
+                                                      const gchar *id);
+
 #endif
