@@ -38,6 +38,7 @@
 #include <gdesktop-enums.h>
 
 #include "cc-rr-labeler.h"
+#include <libupower-glib/upower.h>
 
 CC_PANEL_REGISTER (CcDisplayPanel, cc_display_panel)
 
@@ -97,6 +98,9 @@ struct _CcDisplayPanelPrivate
 
   /* These are used while we are waiting for the ApplyConfiguration method to be executed over D-bus */
   GDBusProxy *proxy;
+
+  UpClient *up_client;
+  gboolean lid_is_closed;
 };
 
 typedef struct
@@ -151,6 +155,10 @@ cc_display_panel_set_property (GObject      *object,
 static void
 cc_display_panel_dispose (GObject *object)
 {
+  CcDisplayPanelPrivate *priv = CC_DISPLAY_PANEL (object)->priv;
+
+  g_clear_object (&priv->up_client);
+
   G_OBJECT_CLASS (cc_display_panel_parent_class)->dispose (object);
 }
 
@@ -693,7 +701,20 @@ rebuild_on_off_radios (CcDisplayPanel *self)
 
   if (!gnome_rr_config_get_clone (self->priv->current_configuration) && self->priv->current_output)
     {
-      if (count_active_outputs (self) > 1 || !gnome_rr_output_info_is_active (self->priv->current_output))
+      gboolean display_closed = FALSE;
+
+      if (self->priv->lid_is_closed)
+        {
+          GnomeRROutput *output;
+
+          output = gnome_rr_screen_get_output_by_name (self->priv->screen,
+                                                       gnome_rr_output_info_get_name (self->priv->current_output));
+
+          display_closed = gnome_rr_output_is_laptop (output);
+        }
+
+      if ((count_active_outputs (self) > 1 || !gnome_rr_output_info_is_active (self->priv->current_output))
+          && !display_closed)
         sensitive = TRUE;
       else
         sensitive = FALSE;
@@ -715,9 +736,9 @@ make_resolution_string (int width, int height)
   const char *aspect = NULL;
 
     /* We use a number of Unicode characters below:
-     * \342\210\266 is U+2236 RATIO
-     * \342\200\211 is U+2009 THIN SPACE,
-     * \303\227\342 is U+00D7 MULTIPLICATION SIGN
+     * ∶ is U+2236 RATIO
+     *   is U+2009 THIN SPACE,
+     * × is U+00D7 MULTIPLICATION SIGN
      */
   if (width && height) {
     if (width > height)
@@ -727,34 +748,37 @@ make_resolution_string (int width, int height)
 
     switch (ratio) {
     case 13:
-      aspect = "4\342\210\2663";
+      aspect = "4∶3";
       break;
     case 16:
-      aspect = "16\342\210\26610";
+      aspect = "16∶10";
       break;
     case 17:
-      aspect = "16\342\210\2669";
+      aspect = "16∶9";
+      break;
+    case 23:
+      aspect = "21∶9";
       break;
     case 12:
-      aspect = "5\342\210\2664";
+      aspect = "5∶4";
       break;
       /* This catches 1.5625 as well (1600x1024) when maybe it shouldn't. */
     case 15:
-      aspect = "3\342\210\2662";
+      aspect = "3∶2";
       break;
     case 18:
-      aspect = "9\342\210\2665";
+      aspect = "9∶5";
       break;
     case 10:
-      aspect = "1\342\210\2661";
+      aspect = "1∶1";
       break;
     }
   }
 
   if (aspect != NULL)
-    return g_strdup_printf ("%d\342\200\211\303\227\342\200\211%d (%s)", width, height, aspect);
+    return g_strdup_printf ("%d × %d (%s)", width, height, aspect);
   else
-    return g_strdup_printf ("%d\342\200\211\303\227\342\200\211%d", width, height);
+    return g_strdup_printf ("%d × %d", width, height);
 }
 
 static void
@@ -1335,6 +1359,12 @@ list_edges_for_output (GnomeRROutputInfo *output, GArray *edges)
 
   gnome_rr_output_info_get_geometry (output, &x, &y, &w, &h);
 
+  if (!gnome_rr_output_info_is_active (output))
+    {
+      h = gnome_rr_output_info_get_preferred_height (output);
+      w = gnome_rr_output_info_get_preferred_width (output);
+    }
+
   apply_rotation_to_geometry (output, &w, &h);
 
   /* Top, Bottom, Left, Right */
@@ -1535,6 +1565,12 @@ get_output_rect (GnomeRROutputInfo *output, GdkRectangle *rect)
 {
   gnome_rr_output_info_get_geometry (output, &rect->x, &rect->y, &rect->width, &rect->height);
 
+  if (!gnome_rr_output_info_is_active (output))
+    {
+      rect->width = gnome_rr_output_info_get_preferred_width (output);
+      rect->height = gnome_rr_output_info_get_preferred_height (output);
+    }
+
   apply_rotation_to_geometry (output, &rect->width, &rect->height);
 }
 
@@ -1738,6 +1774,13 @@ set_primary_output (CcDisplayPanel *self,
     gnome_rr_output_info_set_primary (outputs[i], outputs[i] == output);
 }
 
+static void
+grab_weak_ref_notify (gpointer  area,
+                      GObject  *object)
+{
+  foo_scroll_area_end_grab (area, NULL);
+}
+
 static void
 on_output_event (FooScrollArea *area,
                  FooScrollAreaEvent *event,
@@ -1780,6 +1823,7 @@ on_output_event (FooScrollArea *area,
 	  gnome_rr_output_info_get_geometry (output, &output_x, &output_y, NULL, NULL);
 
 	  foo_scroll_area_begin_grab (area, on_output_event, data);
+	  g_object_weak_ref (data, grab_weak_ref_notify, area);
 
 	  info = g_new0 (GrabInfo, 1);
 	  info->grab_x = event->x;
@@ -1818,7 +1862,7 @@ on_output_event (FooScrollArea *area,
 
 	  g_array_sort (snaps, compare_snaps);
 
-	  gnome_rr_output_info_set_geometry (output, new_x, new_y, width, height);
+	  gnome_rr_output_info_set_geometry (output, old_x, old_y, width, height);
 
 	  for (i = 0; i < snaps->len; ++i)
 	    {
@@ -1852,6 +1896,7 @@ on_output_event (FooScrollArea *area,
 
 	      g_free (g_object_get_data (G_OBJECT (output), "grab-info"));
 	      g_object_set_data (G_OBJECT (output), "grab-info", NULL);
+	      g_object_weak_unref (data, grab_weak_ref_notify, area);
 
 #if 0
               g_debug ("new position: %d %d %d %d", output->x, output->y, output->width, output->height);
@@ -2575,6 +2620,23 @@ cc_display_panel_init (CcDisplayPanel *self)
   g_resources_register (cc_display_get_resource ());
 }
 
+static void
+cc_display_panel_up_client_changed (UpClient       *client,
+                                    CcDisplayPanel *self)
+{
+  CcDisplayPanelPrivate *priv = self->priv;
+  gboolean lid_is_closed;
+
+  lid_is_closed = up_client_get_lid_is_closed (client);
+
+  if (lid_is_closed != priv->lid_is_closed)
+    {
+      priv->lid_is_closed = lid_is_closed;
+
+      rebuild_on_off_radios (self);
+    }
+}
+
 static GObject *
 cc_display_panel_constructor (GType                  gtype,
                               guint                  n_properties,
@@ -2679,5 +2741,21 @@ cc_display_panel_constructor (GType                  gtype,
   gtk_widget_show (self->priv->panel);
   gtk_container_add (GTK_CONTAINER (self), self->priv->panel);
 
+  self->priv->up_client = up_client_new ();
+  if (up_client_get_lid_is_present (self->priv->up_client))
+    {
+      /* Connect to the "changed" signal to track changes to "lid-is-closed"
+       * property. Connecting to "notify::lid-is-closed" would be preferable,
+       * but currently doesn't work as expected:
+       * https://bugs.freedesktop.org/show_bug.cgi?id=43001
+       */
+
+      g_signal_connect (self->priv->up_client, "changed",
+                        G_CALLBACK (cc_display_panel_up_client_changed), self);
+      cc_display_panel_up_client_changed (self->priv->up_client, self);
+    }
+  else
+    g_clear_object (&self->priv->up_client);
+
   return obj;
 }
