@@ -733,6 +733,47 @@ cc_sharing_panel_check_schema_available (CcSharingPanel *self,
   return FALSE;
 }
 
+static void
+screen_sharing_show_cb (GtkWidget *widget, CcSharingPanel *self)
+{
+  CcSharingPanelPrivate *priv = self->priv;
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("show-password-checkbutton")),
+                                FALSE);
+}
+
+#define MAX_PASSWORD_SIZE 8
+static void
+screen_sharing_password_insert_text_cb (GtkEditable *editable,
+                                        gchar       *new_text,
+                                        gint         new_text_length,
+                                        gpointer     position,
+                                        gpointer     user_data)
+{
+  int l, available_size;
+
+  l = gtk_entry_buffer_get_bytes (gtk_entry_get_buffer (GTK_ENTRY (editable)));
+
+  if (l + new_text_length <= MAX_PASSWORD_SIZE)
+    return;
+
+  g_signal_stop_emission_by_name (editable, "insert-text");
+  gtk_widget_error_bell (GTK_WIDGET (editable));
+
+  available_size = g_utf8_strlen (new_text, MAX_PASSWORD_SIZE - l);
+  if (available_size == 0)
+    return;
+
+  g_signal_handlers_block_by_func (editable,
+                                   (gpointer) screen_sharing_password_insert_text_cb,
+                                   user_data);
+  gtk_editable_insert_text (editable, new_text, available_size, position);
+  g_signal_handlers_unblock_by_func (editable,
+                                     (gpointer) screen_sharing_password_insert_text_cb,
+                                     user_data);
+}
+#undef MAX_PASSWORD_SIZE
+
 static void
 cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
 {
@@ -743,17 +784,11 @@ cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
                                          WID ("screen-sharing-status-label"));
 
   cc_sharing_panel_bind_switch_to_widgets (WID ("remote-view-switch"),
-                                           WID ("remote-control-switch"),
-                                           WID ("remote-control-label"),
-                                           WID ("remote-control-grid"),
+                                           WID ("remote-control-box"),
                                            NULL);
 
   cc_sharing_panel_bind_switch_to_widgets (WID ("remote-control-switch"),
-                                           WID ("approve-all-connections-switch"),
-                                           WID ("remote-control-require-password-switch"),
-                                           WID ("remote-control-require-password-label"),
-                                           WID ("approve-all-connections-label"),
-                                           WID ("password-grid"),
+                                           WID ("remote-control-grid"),
                                            NULL);
 
   cc_sharing_panel_bind_switch_to_widgets (WID ("remote-control-require-password-switch"),
@@ -789,6 +824,14 @@ cc_sharing_panel_setup_screen_sharing_dialog (CcSharingPanel *self)
   g_object_bind_property (WID ("show-password-checkbutton"), "active",
                           WID ("remote-control-password-entry"), "visibility",
                           G_BINDING_SYNC_CREATE);
+
+  /* make sure the password entry is hidden by default */
+  g_signal_connect (priv->screen_sharing_dialog, "show",
+                    G_CALLBACK (screen_sharing_show_cb), self);
+
+  /* accept at most 8 bytes in password entry */
+  g_signal_connect (WID ("remote-control-password-entry"), "insert-text",
+                    G_CALLBACK (screen_sharing_password_insert_text_cb), self);
 }
 
 static void
