@@ -149,6 +149,10 @@ user_added (ActUserManager *um, ActUser *user, CcUserPanelPrivate *d)
         GtkTreeSelection *selection;
         gint sort_key;
 
+        if (act_user_is_system_account (user)) {
+                return;
+        }
+
         g_debug ("user added: %d %s\n", act_user_get_uid (user), get_real_or_user_name (user));
         widget = get_widget (d, "list-treeview");
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
@@ -347,6 +351,7 @@ select_created_user (GObject *object,
         ActUser *current;
         GtkTreePath *path;
         ActUser *user;
+        uid_t user_uid;
 
         dialog = UM_ACCOUNT_DIALOG (object);
         user = um_account_dialog_finish (dialog, result);
@@ -359,20 +364,22 @@ select_created_user (GObject *object,
         tv = (GtkTreeView *)get_widget (d, "list-treeview");
         model = gtk_tree_view_get_model (tv);
         selection = gtk_tree_view_get_selection (tv);
+        user_uid = act_user_get_uid (user);
 
         gtk_tree_model_get_iter_first (model, &iter);
         do {
                 gtk_tree_model_get (model, &iter, USER_COL, &current, -1);
-                if (user == current) {
-                        path = gtk_tree_model_get_path (model, &iter);
-                        gtk_tree_view_scroll_to_cell (tv, path, NULL, FALSE, 0.0, 0.0);
-                        gtk_tree_selection_select_path (selection, path);
-                        gtk_tree_path_free (path);
+                if (current) {
+                        if (user_uid == act_user_get_uid (current)) {
+                                path = gtk_tree_model_get_path (model, &iter);
+                                gtk_tree_view_scroll_to_cell (tv, path, NULL, FALSE, 0.0, 0.0);
+                                gtk_tree_selection_select_path (selection, path);
+                                gtk_tree_path_free (path);
+                                g_object_unref (current);
+                                break;
+                        }
                         g_object_unref (current);
-                        break;
                 }
-                if (current)
-                        g_object_unref (current);
         } while (gtk_tree_model_iter_next (model, &iter));
 
         g_object_unref (user);
@@ -1433,6 +1440,10 @@ cc_user_panel_dispose (GObject *object)
 {
         CcUserPanelPrivate *priv = UM_USER_PANEL (object)->priv;
 
+        if (priv->um) {
+                g_signal_handlers_disconnect_by_data (priv->um, priv);
+                priv->um = NULL;
+        }
         if (priv->builder) {
                 g_object_unref (priv->builder);
                 priv->builder = NULL;
