@@ -43,14 +43,8 @@ G_BEGIN_DECLS
  *
  * use: CC_PANEL_REGISTER (PluginName, plugin_name)
  */
-#define CC_PANEL_REGISTER(PluginName, plugin_name)                             \
-        G_DEFINE_DYNAMIC_TYPE (PluginName,                                     \
-                               plugin_name,                                    \
-                               CC_TYPE_PANEL)                                  \
-static void                                                                    \
-plugin_name##_class_finalize (PluginName##Class *plugin_name##_class)          \
-{                                                                              \
-}
+#define CC_PANEL_REGISTER(PluginName, plugin_name)                      \
+  G_DEFINE_TYPE (PluginName, plugin_name, CC_TYPE_PANEL)
 
 typedef struct CcPanelPrivate CcPanelPrivate;
 
