@@ -630,7 +630,8 @@ search_entry_key_press_event_cb (GtkEntry        *entry,
 {
   CcWindowPrivate *priv = self->priv;
 
-  if (event->keyval == GDK_KEY_Return)
+  if (event->keyval == GDK_KEY_Return &&
+      g_strcmp0 (priv->filter_string, "") != 0)
     {
       GtkTreePath *path;
       GtkTreeSelection *selection;
@@ -1070,6 +1071,10 @@ set_active_panel (CcWindow *shell,
         {
           shell->priv->active_panel = g_object_ref (panel);
         }
+      else
+        {
+          shell_show_overview_page (shell);
+        }
       g_object_notify (G_OBJECT (shell), "active-panel");
     }
 }
